/* -*- Mode: java; tab-width: 2; c-basic-offset: 2 -*- */
/*
  merge_pdf_fdf, version 1.0
  merge an input PDF file with an input FDF file
  to create a filled-in PDF; optionally flatten the
  FDF data so that it becomes part of the page
  http://www.pdfhacks.com/merge_pdf_fdf/

  invoke from the command line like this:

    java -classpath ./itext-paulo.jar:. \
    merge_pdf_fdf input.pdf input.fdf output.pdf

  or:

    java -classpath ./itext-paulo.jar:. \
    merge_pdf_fdf input.pdf input.fdf output.pdf flatten

  adjust the classpath to the location of your iText jar
 */

import java.io.*;
import com.lowagie.text.pdf.*;

public class merge_pdf_fdf extends java.lang.Object {
    
  public static void main(String args[]) {
    if ( args.length == 3 || args.length == 4 ) {
      try {
        // the input PDF
        PdfReader reader = 
          new PdfReader( args[0] );
        reader.consolidateNamedDestinations();
        reader.removeUnusedObjects();

        // the input FDF
        FdfReader fdf_reader= 
          new FdfReader( args[1] );

        // PdfStamper acts like a PdfWriter
        PdfStamper pdf_stamper= 
          new PdfStamper( reader,
                          new FileOutputStream( args[2] ) );

        if( args.length == 4 ) { // "flatten"
          // filled-in data becomes a permanent part of the page
          pdf_stamper.setFormFlattening( true );
        }
        else {
          // filled-in data will 'stick' to the form fields,
          // but it will remain interactive
          pdf_stamper.setFormFlattening( false );
        }

        // sets the form fields from the input FDF
        AcroFields fields=
          pdf_stamper.getAcroFields();
        fields.setFields( fdf_reader );

        // closing the stamper closes the underlying
        // PdfWriter; the PDF document is written
        pdf_stamper.close();
      }
      catch( Exception ee ) {
        ee.printStackTrace();
      }
    }
    else { // input error
      System.err.println("arguments: file1.pdf file2.fdf destfile [flatten]");
    }
  }
}
