/* -*- Mode: java; tab-width: 2; c-basic-offset: 2 -*- */
/*
  watermark_pdf, version 1.0
  visit: www.pdfhacks.com/watermark/

  place a single "watermark" PDF page "underneath" all
  pages of the input document

  invoke from the command line like this:

    java -classpath ./itext-paulo.jar:. \
    watermark_pdf doc.pdf watermark.pdf output.pdf

  only the first page of watermark.pdf is used
*/

import java.io.*;
import com.lowagie.text.*;
import com.lowagie.text.pdf.*;

public class watermark_pdf {

  public static void main( String[] args ) {
    if( args.length== 3 ) {
      try {
        // the document we're watermarking
        PdfReader document= new PdfReader( args[0] );
        int num_pages= document.getNumberOfPages();

        // the watermark (or letterhead, etc.)
        PdfReader mark= new PdfReader( args[1] );
        Rectangle mark_page_size= mark.getPageSize( 1 );

        // the output document
        PdfStamper writer= 
          new PdfStamper( document, 
                          new FileOutputStream( args[2] ) );

        // create a PdfTemplate from the first page of mark
        // (PdfImportedPage is derived from PdfTemplate)
        PdfImportedPage mark_page=
          writer.getImportedPage( mark, 1 );

        for( int ii= 0; ii< num_pages; ) {
          // iterate over document's pages, adding mark_page as
          // a layer 'underneath' the page content; scale mark_page
          // and move it so it fits within the document's page;
          // if document's page is cropped, then this scale might
          // not be small enough

          ++ii;
          Rectangle doc_page_size= document.getPageSize( ii );
          float h_scale= doc_page_size.width() / mark_page_size.width();
          float v_scale= doc_page_size.height() / mark_page_size.height();
          float mark_scale= (h_scale< v_scale) ? h_scale :  v_scale;

          float h_trans= (float)((doc_page_size.width()- 
                                  mark_page_size.width()* mark_scale) / 2.0);
          float v_trans= (float)((doc_page_size.height()- 
                                  mark_page_size.height()* mark_scale) / 2.0);
          
          PdfContentByte contentByte= writer.getUnderContent( ii );
          contentByte.addTemplate( mark_page, 
                                   mark_scale, 0, 
                                   0, mark_scale, 
                                   h_trans, v_trans );
        }

        writer.close();
      }
      catch( Exception ee ) {
        ee.printStackTrace();
      }
    }
    else { // input error
      System.err.println("arguments: in_document in_watermark out_pdf_fn");
    }
  }
}
